--[[
Copyright (C) 2016  Neemys, ShadowTheAge

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

See <LICENSE.txt> for a copy of this license, or see <http://www.gnu.org/licenses/>
]]



if not buildingplatform.tiles then buildingplatform.tiles = {} end

local howManyCall = 0 --lua don't have static, so at least keep track using a local to file variable
local layerList = {"layer-15","layer-14","layer-13","layer-12"} --take the far most layer first in case the game use up some layer, it will not mess with most use of our function and the mod will continue to work until it is updated.


--[[
The following function should be called when you want to add a tile that can have blocking/allowing buildings rules.

TODO : make platform that have no rules to don't take a layer.
TODO : parameter verification

tileName : string, name that should be given to the tile, used in tile definition and noise layer name (if needed)
initAutoplace : boolean, if true, then the autoplace object will be added to the tile, autoplace-control and noise-layer will be added to data. If you don't use the game's default resource placement then you need to set this to false.
tileAutoplaceOrder : string, factorio formated order to tell the game where to put the autoplace config line in new world screen (only if initAutoplace is true).
tileAutoplaceObject : factorio autoplace object to add to the tile (only if initAutoplace is true).
tilePicturesDirectory : string, location of all picture that represent the platform tile. Name of each picture should match factorio pictures names whose are : [tile-name]1.png, [tile-name]2.png, [tile-name]4.png,  [tile-name]-inner-corner.png, [tile-name]-o.png, [tile-name]-outer-corner.png, [tile-name]-side.png, [tile-name]-u.png and [tile-name]-icon.png (for the item). If for a particular reason you have pictures in multiple different folder or pictures that do not match factorio pictures name, use the function (tileFunctionToEditTileDefinition) to edit those pictures location. Do not need to add a slash at the end of the directory.
tileMapColor : factorio color object, map_color object a tile definition need.
tileFunctionToEditTileDefinition : function with one parameter : the factorio tile object created for the platform. If you need to do specific modification to the tile definition use this function. This function is called after the tile definition is populated by my mod but before it is added to factorio data. You can't edit the tile name here.

return a table (tileObject) that can be given to other function of my mod to add rules to this platform. You shouldn't edit that object by yourself.
In case you need that object somewhere where you can't pass the object returned, you can get it in buildingplatform.tiles[yourTileName]. 
]]


buildingplatform.addPlatform = function(tileName,initAutoplace,tileAutoplaceOrder,tileAutoplaceObject,tilePicturesDirectory,tileMapColor,tileFunctionToEditTileDefinition)
	howManyCall = howManyCall + 1
	if howManyCall >= 5 then
		debug.traceback("From building-platform mod, you can't add more than 4 platforms for the moment, you can find the fifth call using the following stacktrace :", 2)
	end
	if buildingplatform.tiles[tileName] then
		debug.traceback("From building-platform mod, trying to add a duplicate platform (same name), find the second call using the following stacktrace", 2)
	end
	tile = {
		name=tileName,
		needToInitAutoplace = initAutoplace,
		autoplaceOrder = tileAutoplaceOrder,
		autoplaceObject = tileAutoplaceObject,
		picturesDirectory = tilePicturesDirectory,
		mapColor = tileMapColor,
		tileFunction = tileFunctionToEditTileDefinition,
		buildingsDisallowed = {},
		buildingsAllowed = {},
		layer=layerList[howManyCall]
	}
	buildingplatform.tiles[tileName] = tile
	return tile
end